(function(window){
    console.log('js init');
    if(window._inject_flag) return;
    if (typeof BugInsightJSBridge == 'undefined') {
        return;
    }
    var doc = document;
    if(doc.URL == ''){
        console.log('document.URL is null');
        return;
    }
    var util = {
        wrap: function(force, obj, name, wpr, oname) {
            try {
                var raw = obj[name];
            } catch (e) {
                if (!force) return false;
            }
            if (!raw && !force) return false;
            if (raw && raw._ty_wrap) return false;
            try {
                obj[name] = wpr(raw, oname);
            } catch (e) {
                return false;
            };
            obj[name]._ty_wrap = [raw];
            return true;
        },
        unwrap: function(obj, name) {
            try {
                var raw = obj[name]._ty_wrap;
                if (raw) {
                    obj[name] = raw[0];
                }
            } catch (e) {}
        },
        each: function each(ary, func) {
            if (ary) {
                var i;
                for (i = 0; i < ary.length; i += 1) {
                    if (ary[i] && func(ary[i], i, ary)) {
                        break;
                    }
                }
            }
        },
        sh: function(obj, evt, handler, f) {
            if (obj.addEventListener) {
                return obj.addEventListener(evt, handler, f);
            } else if (obj.attachEvent) {
                return obj.attachEvent('on' + evt, handler);
            }
            return false;
        },
        stringify: stringify,
        trim: trim ?
            function(text) {
                return text == null ?
                    "" :
                    trim.call(text);
            } : function(text) {
                return text == null ?
                    "" :
                    text.toString().replace(/^\s+/, "").replace(/\s+$/, "");
            },
        extend: function(target, source) {
            if (target && source) {
                for (var key in source) {
                    if (source.hasOwnProperty(key)) {
                        target[key] = source[key];
                    }
                }
            }
            return target;
        },
        bind: function(func, context) {
            return function() {
                return func.apply(context, arguments);
            }
        }
    };

    var bugInsight_web = window._inject_flag = {
        st: now(),
        ra: [],
    };

    console.log('step1');
    var trim = String.prototype.trim;
    if (!String.prototype.startsWith) {
        String.prototype.startsWith = function(searchString, position) {
            position = position || 0;
            return this.indexOf(searchString, position) === position;
        };
    }

    function stringify(value) {
        switch (typeof value) {
            case 'object':
                if (!value) {
                    return 'null';
                }
                if (value instanceof Array) {
                    var result = '[';
                    for (var i = 0; i < value.length; i++) {
                        result += ((i > 0) ? ',' : '') + stringify(value[i]);
                    }
                    return result + ']';
                }
                if (value instanceof Date) {
                    return value.getTime().toString();
                }
                var result = '{';
                var i = 0;
                for (var key in value) {
                    if (typeof value[key] === 'function') continue;
                    var val = stringify(value[key]);
                    result += ((i > 0) ? ',' : '') + stringify(key) + ':' + val;
                    i++;
                }
                return result + '}';
            case 'string':
                return '\"' + value.replace(/([\"\\])/g, '\\$1').replace(/\n/g, '\\n') + '\"';
            case 'number':
                return value.toString();
            case 'boolean':
                return value ? 'true' : 'false';
            case 'function':
                return stringify(value.toString());
            case 'undefined':
            default:
                return '\"undefined\"';
        }
    };

    function now() {
        return Date.now ? Date.now() : (new Date()).valueOf();
    };

    var performance = window.performance ? window.performance : window.Performance;
    if (performance) {
        util.sh(performance, 'resourcetimingbufferfull', function() {
            var rcs = performance.getEntriesByType("resource");
            if (!rcs) {
                return;
            }
            bugInsight_web.ra = bugInsight_web.ra.concat(rcs);
            performance.clearResourceTimings();
        }, false);
        util.sh(performance, 'webkitresourcetimingbufferfull', function() {
            var rcs = performance.getEntriesByType("resource");
            if (!rcs) {
                return;
            }
            bugInsight_web.ra = bugInsight_web.ra.concat(rcs);
            performance.webkitClearResourceTimings();
        }, false);
    }

    var metric = {
        ready: function() {
            return bugInsight_web.load_time;
        },
        send: function() {
            console.log('call send');
            if (this.sended) {
                return false;
            }
            if (!this.ready()) {
                return false;
            }
            var s = bugInsight_web.st;

            function get_pageurl_metric() {
                console.log('call get_pageurl_metric');
                var r = {};
                if (performance && performance.timing) {
                    var timing = performance.timing;
                    navigationStart = timing.navigationStart;
                    result = {
                        navigationStart: navigationStart,
                        url: doc.URL,
                        fetchStart: timing["fetchStart"] - navigationStart,
                        connectStart: timing["connectStart"] - navigationStart,
                        connectEnd: timing["connectEnd"] - navigationStart,
                        domainLookupStart: timing["domainLookupStart"] - navigationStart,
                        domainLookupEnd: timing["domainLookupEnd"] - navigationStart,
                        requestStart: timing["requestStart"] - navigationStart,
                        responseStart: timing["responseStart"] - navigationStart,
                        responseEnd: timing["responseEnd"] - navigationStart,
                        domContentLoadedEventStart: timing["domContentLoadedEventStart"] - navigationStart,
                        domContentLoadedEventEnd: timing["domContentLoadedEventEnd"] - navigationStart,
                        domInteractive: timing["domInteractive"] - navigationStart,
                        domComplete:    timing["domComplete"] - navigationStart,
                        domLoading:    timing["domLoading"] - navigationStart,
                        loadEventStart:    timing["loadEventStart"] - navigationStart,
                        loadEventEnd:    timing["loadEventEnd"] - navigationStart,
                        unloadEventStart:    timing["unloadEventStart"] - navigationStart,
                        unloadEventEnd:    timing["unloadEventEnd"] - navigationStart
                    };

                    if (timing.secureConnectionStart) {
                        result.secureConnectionStart = timing["secureConnectionStart"] - navigationStart;
                    }

                    console.log(result);
                    return result;
                }
            }

            function get_navigation() {
                console.log('get navigation info');
                var pf = performance;
            }

            function get_rc_array() {
                var pf = performance;
                console.log('call get_rc_array');
                if (pf && pf.getEntriesByType) {
                    var r = {};
                    var rc = bugInsight_web.ra;
                    var rcs = pf.getEntriesByType("resource");
                    if (rcs) {
                        rc = rc.concat(rcs);
                        if (pf.clearResourceTimings) {
                            pf.clearResourceTimings();
                        } else if (pf.webkitClearResourceTimings) {
                            pf.webkitClearResourceTimings();
                        }
                    }
                    r.res = [];
                    for (var i = 0; i < rc.length; i++) {
                        var t = rc[i];

                        function v(n) {
                            return (t[n] > 0) ? t[n] : 0;
                        }
                        var item = {
                            startTime: v("startTime"),
                            initiatorType: t.initiatorType,
                            name: t.name,
                            duration: v("duration"),
                            fetchStart: v("fetchStart"),
                            domainLookupStart: v("domainLookupStart"),
                            domainLookupEnd: v("domainLookupEnd"),
                            connectStart: v("connectStart"),
                            connectEnd: v("connectEnd"),
                            secureConnectionStart: v("secureConnectionStart"),
                            requestStart: v("requestStart"),
                            responseStart: v("responseStart"),
                            responseEnd: v("responseEnd"),
                            transferSize: t.transferSize || 0,
                            encodedBodySize: t.encodedBodySize || 0,
                            decodedBodySize:t.decodedBodySize||0
                        };
                        console.log('push item:' + item);
                        r.res.push(item);
                    }
                    return r;
                }
                return null;
            }

            var result = {};
            var pf = get_pageurl_metric();
            if (pf.domComplete <= 0 || pf.loadEventEnd <= 0) {
                console.log('invalid performance data');
                return;
            }
            console.log('page url info:' + util.stringify(pf));

            result = get_rc_array();
            result = result ? util.stringify(result) : '';

            var dataCombine = {};
            dataCombine.pageres = result;

            if ((pf.url.startsWith("http") || pf.url.startsWith("file:"))) {
                pf.header = "";
                dataCombine.pageurl = pf ? util.stringify(pf) : '';
                console.log('send data to java');
                BugInsightJSBridge.logJsResult(dataCombine ? util.stringify(dataCombine) : '');
            }

            // var postError = util.bind(sendError, this);
            // postError();
            // //设置每隔10s发送一次error
            // setInterval(postError, 10000);

            this.sended = true;
            return true;
        },
        errors: []
    };

    function onDocLoaded() {
    };

    function onReadyStateChange() {
        if (doc.readyState === 'complete') {
        }
    };

    function onWindowLoad(a) {
        console.log('onWindowLoad, status:%d', a);
        if (bugInsight_web.load_time) {
//            return true;
        }
        bugInsight_web.load_time = now();

        function cb() {
            metric.send();
        };
        if (a === 9) {
            cb();
        } else {
            setTimeout(cb, 0);
        }
    };

    var ul = null;
    var isFromUnLoad = false;
    function onWindowUnload() {
        if (!ul) {
            isFromUnLoad = true;
            onWindowLoad(9);
        }
        ul = 1;
    };

    var w_evts = [
        ['load', onWindowLoad],
        ['beforeunload', onWindowUnload],
        ['pagehide', onWindowUnload],
        ['unload', onWindowUnload]
    ];

    for (var i = 0; i < w_evts.length; i++) {
        util.sh(window, w_evts[i][0], w_evts[i][1], false)
    }

    var d_evts = [
        ['DOMContentLoaded', onDocLoaded],
        ['readystatechange', onReadyStateChange]
    ];
    for (var i = 0; i < d_evts.length; i++) {
        util.sh(doc, d_evts[i][0], d_evts[i][1], false)
    }

    function onErrorEvent(obj) {
        console.log("onErrorEvent happened")
        var a = arguments,
            ud = "unknown",
            p = [now()];
        if (a.length == 0) {
            return;
        }
        if (typeof obj === 'string') {
        } else if (obj instanceof Event || (window.ErrorEvent && obj instanceof ErrorEvent)) {
            p[1] = obj.message || ((obj.error && obj.error["constructor"].name) + (obj.error && obj.error.message)) || "";
            p[2] = obj.lineno ? obj.lineno : 0;
            p[3] = obj.colno ? obj.colno : 0;
            p[4] = obj.filename || (obj.error && obj.error.fileName) || (obj.target && obj.target.baseURI) || "";
            if (!p[4]) {
                console.log('on error: filename is empty and ignored');
                return;
            }
            if (p[4] == doc.URL) {
                p[4] = '#';
            }
            if (obj.error) {
                p[5] = obj.error.stack;
                p[6] = obj.error.moduleName;
            } else {
                p[5] = null;
                p[6] = null;
            }
            // var errorKey = errorId(p[1], p[2], p[3], p[6]);
            //p[7] = errorTypeCache[errorKey] ? 0 : 1;//这里是表示是否是重复的url吧~~
            //改成手机端需要的
            p[7] = 0;
            // if(!isFirstError){
            //     p[7] = 1; //该js错误是否是当前页面的第一个js错误
            //     isFirstError = true;
            // }
            // errorTypeCache[errorKey] = true;
            if (p[1] === ud && p[4] === ud) {
                return;
            }
            fixInfo(p);

            if (obj.error) {
                p[8] = obj.error.name;
            } else {
                p[8] = null;
            }
            p[9] = now() - getStartTime(); //错误发生时间
        }
        metric.errors.push(p);
    }

    function fixInfo(p) {
        // only fix stack info when there is a module name.
        if (p[6]) {
            var filename = p[4];
            var stack = p[5];
            if (stack && typeof stack === 'string' && filename) {
                stack = stack.split(/\n/);
                var result = URL_REG.exec(stack[0]);
                if (!result) {
                    result = URL_REG.exec(stack[1]);
                }
                if (result && result[1] != filename) {
                    p[4] = result[1] || filename;
                    p[2] = result[2] || p[2];
                    p[3] = result[3] || p[3];
                }
            }
        }
    }

    function errorId(msg, lineno, colno, module) {
        return String(msg) + String(lineno) + String(colno) + String(module);
    }

    function getStartTime() {
        return performance.timing && performance.timing.navigationStart || bugInsight_web.st;
    }

    function sendError() {
        console.log("metric errors length:" + metric.errors.length);
        if (metric.errors.length) {
            var errors = metric.errors;
            // var that = this;
            //考虑到error是分为不同的部分传输的，分为dom load之前和之后，此是在之后传输的，
            //所以分成了2个函数了，这个sendError是在之后进行传输
            try {
                console.log('after dom error:' + (errors ? util.stringify(errors) : ''));
                util.each(errors, function (err) {
                    // nbsJsBridge.logJsError(guid, doc.URL, err[8], err[0],
                    //     err[1], err[2], err[3], err[5], err[4], err[7], "", getStartTime(), err[9]);
                    BugInsightJSBridge.logJsError("guid", doc.URL, err[8], err[0],
                        err[1], err[2], err[3], err[5], err[4], err[7], "", getStartTime(), err[9]);
                });
            } catch (e) {
            }
            // errors.errors = [];
            metric.errors = [];
        }
    }
    if (window.addEventListener) {
        util.sh(window, 'error', onErrorEvent, false);
    }

    if(!this.setJsErrorSend){
        //js的独立出来发送吧，有时候send函数没有被调用
        var postError = util.bind(sendError, this);
        postError();
        //设置每隔10s发送一次error
        setInterval(postError, 10000);
        this.setJsErrorSend = true;
    }

})(window)